"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const react_1 = __importDefault(require("react"));
const server_1 = require("react-dom/server");
const defaultComponentDecorator_1 = __importDefault(require("../decorators/defaultComponentDecorator"));
const defaultHrefDecorator_1 = __importDefault(require("../decorators/defaultHrefDecorator"));
const defaultMatchDecorator_1 = __importDefault(require("../decorators/defaultMatchDecorator"));
const defaultTextDecorator_1 = __importDefault(require("../decorators/defaultTextDecorator"));
const Linkify = (props) => {
    const { matchDecorator = defaultMatchDecorator_1.default, componentDecorator = defaultComponentDecorator_1.default, textDecorator = defaultTextDecorator_1.default, hrefDecorator = defaultHrefDecorator_1.default, children, renderAs = "child" } = props;
    function parseString(string) {
        if (string === "") {
            return string;
        }
        const matches = matchDecorator(string);
        if (!matches) {
            return string;
        }
        const elements = [];
        let lastIndex = 0;
        matches.forEach((match, i) => {
            // Push preceding text if there is any
            if (match.index > lastIndex) {
                elements.push(string.substring(lastIndex, match.index));
            }
            const decoratedHref = hrefDecorator(match.url);
            const decoratedText = textDecorator(match.text);
            const decoratedComponent = componentDecorator(decoratedHref, decoratedText, i);
            elements.push(decoratedComponent);
            lastIndex = match.lastIndex;
        });
        // Push remaining text if there is any
        if (string.length > lastIndex) {
            elements.push(string.substring(lastIndex));
        }
        return (elements.length === 1) ? elements[0] : elements;
    }
    function parse(children, key = 0) {
        if (typeof children === "string") {
            return parseString(children);
        }
        else if (react_1.default.isValidElement(children) && (children.type !== "a") && (children.type !== "button")) {
            return react_1.default.cloneElement(children, { key: key }, parse(children.props.children));
        }
        else if (Array.isArray(children)) {
            return children.map((child, i) => parse(child, i));
        }
        return children;
    }
    if (renderAs === "child") {
        return (react_1.default.createElement(react_1.default.Fragment, null, parse(children)));
    }
    else {
        // Support react components as innerHTML
        const content = parse(children);
        const stringifiedContent = htmlDecode((0, server_1.renderToString)(content));
        return (react_1.default.createElement("div", { dangerouslySetInnerHTML: { __html: stringifiedContent } }));
    }
};
function htmlDecode(input) {
    const e = document.createElement("textarea");
    e.innerHTML = input;
    // handle case of empty input
    return e.childNodes.length === 0 ? "" : e.childNodes[0].nodeValue;
}
exports.default = Linkify;
