import React from "react";
import { renderToString } from "react-dom/server";
import defaultComponentDecorator from "../decorators/defaultComponentDecorator";
import defaultHrefDecorator from "../decorators/defaultHrefDecorator";
import defaultMatchDecorator from "../decorators/defaultMatchDecorator";
import defaultTextDecorator from "../decorators/defaultTextDecorator";
const Linkify = (props) => {
    const { matchDecorator = defaultMatchDecorator, componentDecorator = defaultComponentDecorator, textDecorator = defaultTextDecorator, hrefDecorator = defaultHrefDecorator, children, renderAs = "child" } = props;
    function parseString(string) {
        if (string === "") {
            return string;
        }
        const matches = matchDecorator(string);
        if (!matches) {
            return string;
        }
        const elements = [];
        let lastIndex = 0;
        matches.forEach((match, i) => {
            // Push preceding text if there is any
            if (match.index > lastIndex) {
                elements.push(string.substring(lastIndex, match.index));
            }
            const decoratedHref = hrefDecorator(match.url);
            const decoratedText = textDecorator(match.text);
            const decoratedComponent = componentDecorator(decoratedHref, decoratedText, i);
            elements.push(decoratedComponent);
            lastIndex = match.lastIndex;
        });
        // Push remaining text if there is any
        if (string.length > lastIndex) {
            elements.push(string.substring(lastIndex));
        }
        return (elements.length === 1) ? elements[0] : elements;
    }
    function parse(children, key = 0) {
        if (typeof children === "string") {
            return parseString(children);
        }
        else if (React.isValidElement(children) && (children.type !== "a") && (children.type !== "button")) {
            return React.cloneElement(children, { key: key }, parse(children.props.children));
        }
        else if (Array.isArray(children)) {
            return children.map((child, i) => parse(child, i));
        }
        return children;
    }
    if (renderAs === "child") {
        return (React.createElement(React.Fragment, null, parse(children)));
    }
    else {
        // Support react components as innerHTML
        const content = parse(children);
        const stringifiedContent = htmlDecode(renderToString(content));
        return (React.createElement("div", { dangerouslySetInnerHTML: { __html: stringifiedContent } }));
    }
};
function htmlDecode(input) {
    const e = document.createElement("textarea");
    e.innerHTML = input;
    // handle case of empty input
    return e.childNodes.length === 0 ? "" : e.childNodes[0].nodeValue;
}
export default Linkify;
